package dict

import (
	"os"
	"reflect"
	"testing"
)

// TestLoadYamlUsingTempFile tests the loadYaml function.
func TestLoadYamlUsingTempFile(t *testing.T) {
	// Step 1: Create a temporary file.
	tmpFile, err := os.CreateTemp("", "example.*.yaml")
	if err != nil {
		t.Fatalf("Failed to create temp file: %v", err)
	}
	defer os.Remove(tmpFile.Name()) // Clean up after the test.

	// Step 2: Write some YAML content to this file.
	yamlContent := `
key1: value1
key2: 2
key3:
  - elem1
  - elem2
`
	if _, err := tmpFile.Write([]byte(yamlContent)); err != nil {
		t.Fatalf("Failed to write to temp file: %v", err)
	}
	if err := tmpFile.Close(); err != nil {
		t.Fatalf("Failed to close temp file: %v", err)
	}

	// Step 3: Use the loadYaml function to load this file into a Dict.
	dict, err := loadYaml(tmpFile.Name())
	if err != nil {
		t.Fatalf("loadYaml failed: %v", err)
	}

	// Step 4: Verify that the Dict contains the expected data.
	expectedDict := Dict{
		"key1": "value1",
		"key2": 2,
		"key3": []interface{}{"elem1", "elem2"},
	}

	if !reflect.DeepEqual(dict, expectedDict) {
		t.Errorf("Expected dict to be %v, got %v", expectedDict, dict)
	}
}

// TestLoadYaml tests the loadYaml function.
func TestLoadYaml(t *testing.T) {

	dict, err := loadYaml("test_data/dict1.yaml")
	if err != nil {
		t.Fatalf("loadYaml() returned an error: %v", err)
	}

	expectedDict := Dict{
		"key1": "value1",
		"key2": 2,
		"key3": []interface{}{"elem1", "elem2"},
	}

	if !reflect.DeepEqual(dict, expectedDict) {
		t.Errorf("Expected dict to be %v, got %v", expectedDict, dict)
	}
}

func TestDumpJsonStrIndent(t *testing.T) {
	d := Dict{"key": "value", "number": 1}
	jsonStr, err := d.dumpJsonStrIndent("", "  ")
	if err != nil {
		t.Fatalf("dumpJsonStrIndent() returned an error: %v", err)
	}

	expected := `{
  "key": "value",
  "number": 1
}`
	if jsonStr != expected {
		t.Errorf("Expected %v, got %v", expected, jsonStr)
	}
}
